(function (document, Drupal) {

  'use strict';

  Drupal.drimage = {};

  Drupal.drimage.findDelayParent = function (el) {
    if (el.parentNode == null) {
      return null;
    }
    if (el.parentNode.classList && el.parentNode.classList.contains('js-delay-drimage')) {
      return el.parentNode;
    }
    return Drupal.drimage.findDelayParent(el.parentNode);
  };

  Drupal.drimage.resize = function (size, r, d) {
    if (size[d] == 0) {
      return size;
    }

    // Clone values into new array.
    var new_size = {
      0: size[0],
      1: size[1]
    };
    new_size[d] = r;

    var inverse_d = Math.abs(d - 1);
    if (size[inverse_d] == 0) {
      return new_size;
    }
    new_size[inverse_d] = Math.round(new_size[inverse_d] * (new_size[d] / size[d]));

    return new_size;
  };

  Drupal.drimage.fetchData = function(el) {
    var data = JSON.parse(el.getAttribute('data-drimage'));
    data.upscale_x = parseInt(data.upscale_x);
    data.upscale_y = parseInt(data.upscale_y);
    data.downscale_x = parseInt(data.downscale_x);
    data.downscale_y = parseInt(data.downscale_y);
    data.threshold = parseInt(data.threshold);
    return data;
  };

  Drupal.drimage.size = function (el) {
    var multiplier = Number(window.devicePixelRatio);
    var height = 0;
    if (isNaN(multiplier) === true || multiplier <= 0) {
      multiplier = 1;
    }
    // Only set the height if parent element has the img-crop class.
    if (el.parentNode.classList && el.parentNode.classList.contains('img-crop')
        || el.parentNode.parentNode.classList && el.parentNode.parentNode.classList.contains('img-crop')) {
      height = el.offsetHeight;
    }

    var size = {
      0: Math.round(el.offsetWidth * multiplier),
      1: Math.round(height * multiplier)
    };
    var data = Drupal.drimage.fetchData(el);

    // Make sure the requested image is between the down/up-scale parameters.
    if (size[0] < data.upscale_x) {
      size = Drupal.drimage.resize(size, data.upscale_x, 0);
    }
    if (size[1] < data.upscale_y) {
      size = Drupal.drimage.resize(size, data.upscale_y, 1);
    }
    if (size[0] > data.downscale_x) {
      size = Drupal.drimage.resize(size, data.downscale_x, 0);
    }
    if (size[1] > data.downscale_y) {
      size = Drupal.drimage.resize(size, data.downscale_y, 1);
    }

    // Reduce all widths to a multiplier of the threshold, starting at the
    // minimal upscaling.
    var w = size[0] - data.upscale_x;
    var r = (Math.ceil(w/data.threshold) * data.threshold) + data.upscale_x;
    size = Drupal.drimage.resize(size, r, 0);

    // @todo: cover an aspect ratio option here

    return size;
  };

  Drupal.drimage.init = function (context) {
    if (context == null) {
      context = document;
    }
    var el = context.querySelectorAll('.img-wrap img');
    if (el.length > 0) {
      for (var i = 0; i < el.length; i++) {
        var delay = Drupal.drimage.findDelayParent(el[i]);
        if (delay == null) {
          Drupal.drimage.renderEl(el[i]);
        }
      }
    }
  };

  Drupal.drimage.renderEl = function (el) {
    var rect = el.getBoundingClientRect();
    if (
      rect.top    >= 0
      && rect.left   >= 0
      && rect.top <= (window.innerHeight || document.documentElement.clientHeight)
    ) {

      var data = Drupal.drimage.fetchData(el);
      if (isNaN(data.fid) === false && data.fid % 1 === 0 && Number(data.fid) > 0) {
        var size = Drupal.drimage.size(el);
        var w = Number(el.getAttribute('data-w'));
        var h = Number(el.getAttribute('data-h'));
        if (size[0] != w || size[1] != h) {
          el.setAttribute('src', '/drimage/' + size[0] + '/' + size[1] + '/' + data.fid + '/' + encodeURIComponent(data.filename));
          el.setAttribute('data-w', size[0]);
          el.setAttribute('data-h', size[1]);
        }
      }
    }
  };

  Drupal.behaviors.drimage = {
    attach: function (context) {
      Drupal.drimage.init(context);
      var timer;
      addEventListener('resize', function () {
        clearTimeout(timer);
        timer = setTimeout(Drupal.drimage.init, 100);
      });
      addEventListener('scroll', function () {
        Drupal.drimage.init(document);
      });
    }
  };

})(document, Drupal);
