<?php

namespace Drupal\drimage\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Template\Attribute;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;

/**
 * Plugin implementation of the 'dynamic responsive image' formatter.
 *
 * @FieldFormatter(
 *   id = "drimage",
 *   label = @Translation("Dynamic Responsive Image"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class DrImageFormatter extends ImageFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
        'image_crop' => 0,
      ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $element = parent::settingsForm($form, $form_state);

    // Do not use a image style here. Drimage calculates one for us.
    unset($element['image_style']);

    $element['image_crop'] = array(
      '#title' => $this->t('Crop image'),
      '#description' => $this->t('When enabling cropping, your theme\'s CSS should define a fixed (px, em, rem) height on the wrapper around around the img-wrap div element. If it does not the height wil be calculated according to your CSS rules and height inheritance.'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('image_crop'),
    );

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary[] = $this->t('Crop image: @image_crop',
      array('@image_crop' => $this->getSetting('image_crop') ? $this->t('Yes') : $this->t('No'))
    );

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    $attributes = new Attribute();
    $attributes['class'] = array('img-wrap');

    $crop = $this->getSetting('image_crop');
    foreach ($elements as $delta => $element) {
      $elements[$delta]['#item_attributes'] = $attributes;
      $elements[$delta]['#crop'] = $crop;
      $elements[$delta]['#theme'] = 'drimage_formatter';
    }

    return $elements;
  }

}
